(*
 * Copyright (c) 2005 Anil Madhavapeddy <anil@recoil.org>
 * Copyright (c) 2010 Alexander Markov <aspheronets@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $Id: atom_schema.ml,v 1.2 2005/09/20 19:09:06 avsm Exp $
 *)

type common_attr = {{ {base=?String lang=?String } }}

module Text = struct
	type enc_val = {{ "text" | "html" | "xhtml" }}
	type attr = {{ {type=?enc_val} }}
end

type id = {{ <id>[PCDATA] }}
type title = {{ <title (Text.attr)>[PCDATA] }}
type updated = {{ <updated>[PCDATA] }}
type feed_req = {{ [title updated id] }}

module Link = struct
	type rel = {{ "alternate"|"enclosure"|"related"|"self"|"via" }}
	type req = {{ { href=String } }}
	type opt = {{ {rel=?rel hreflang=?String type=?String length=?String
		title=?String} }}
	type attrs = {{ req ++ opt }}
end

module Author = struct
	type name = {{ <name>[PCDATA] }}
	type email = {{ <email>[PCDATA] }}
	type uri = {{ <uri>[PCDATA] }}
	type elem = {{ [name email? uri?] }}
end

type category = {{ <category
	({term=String scheme=?String label=?String})>[PCDATA]}}
type generator = {{ <generator
	({uri=?String version=?String})>[PCDATA] }}
type author = {{ <author>Author.elem }}
type contributor = {{ <contributor>Author.elem }}
type link = {{ <link (Link.attrs)>[] }}
type icon = {{ <icon>[PCDATA] }}
type logo = {{ <logo>[PCDATA] }}

type rights = {{ <rights (Text.attr)>[PCDATA] }}
type subtitle = {{ <subtitle (Text.attr)>[PCDATA] }}

module Entry = struct
    type data = {{ [Xhtml_types_duce._div | Char*] }}
    type content = {{ <content (Text.attr)>data }}
    type summary = {{ <summary (Text.attr)>[PCDATA] }}
    type published = {{ <published>[PCDATA] }}
    type source = {{ <source common_attr>[PCDATA] }}
    type t = {{ <entry>[summary title updated id author* category* content? contributor* link* published? rights? source?] }}
end

type feed_opt = {{ [author? contributor? link* generator?
	icon? logo? rights? subtitle? category* Entry.t*] }}

type feed_all = {{ feed_req @ feed_opt }}
type feed = {{ <feed xmlns=String>feed_all }}
